<?php

namespace Phalcon\Test\Unit\Mvc\Model;

use Phalcon\Mvc\Model\Manager;
use Phalcon\Test\Models\AlbumORama\Albums;
use Phalcon\Test\Module\UnitTest;
use Phalcon\Test\Models\Customers;

/**
 * \Phalcon\Test\Unit\Mvc\Model\ManagerTest
 * Tests the Phalcon\Mvc\Model\Manager component
 *
 * @copyright (c) 2011-2016 Phalcon Team
 * @link      http://www.phalconphp.com
 * @author    Andres Gutierrez <andres@phalconphp.com>
 * @author    Serghei Iakovlev <serghei@phalconphp.com>
 * @author    Wojciech Ślawski <jurigag@gmail.com>
 * @package   Phalcon\Test\Unit\Mvc\Model
 *
 * The contents of this file are subject to the New BSD License that is
 * bundled with this package in the file docs/LICENSE.txt
 *
 * If you did not receive a copy of the license and are unable to obtain it
 * through the world-wide-web, please send an email to license@phalconphp.com
 * so that we can send you a copy immediately.
 */
class ManagerTest extends UnitTest
{
    /**
     * @var Manager
     */
    private $modelsManager;

    protected function _before()
    {
        parent::_before();

        /** @var \Phalcon\Mvc\Application $app */
        $app = $this->tester->getApplication();
        $this->modelsManager = $app->getDI()->getShared('modelsManager');
    }

    public function testAliasedNamespacesRelations()
    {
        $this->specify(
            "Aliased namespaces should work in relations",
            function () {
                $this->modelsManager->registerNamespaceAlias('AlbumORama','Phalcon\Test\Models\AlbumORama');
                $albums = Albums::find();
                foreach($albums as $album){
                    expect($album->artist)->isInstanceOf('Phalcon\Test\Models\AlbumORama\Artists');
                }
            }
        );
    }

    /**
     * Tests Manager::isVisibleModelProperty
     *
     * @author Serghei Iakovlev <serghei@phalconphp.com>
     * @since  2016-08-12
     */
    public function testModelPublicProperties()
    {
        $this->specify(
            'The Manager::isVisibleModelProperty does not check public property correctly',
            function ($property, $expected) {
                expect($this->modelsManager->isVisibleModelProperty(new Customers, $property))->equals($expected);
            }, ['examples' => [
                ['id', true],
                ['document_id', true],
                ['customer_id', true],
                ['first_name', true],
                ['some_field', false],
                ['', false],
                ['protected_field', false],
                ['private_field', false],
            ]]
        );
    }
}
